# Main model for the effect of cond and dB 

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(brms)
library(ggplot2)
source('in1_helperFun.R')

df = read.csv('../data/in1.csv', stringsAsFactors = TRUE)

table(df$cond, df$dB)
aggregate(respNorm ~ scale, df, range)
nrow(df) / length(unique(df$sound)) / length(unique(df$scale))  # 6.7
ratingsPerSound = aggregate(response ~ sound + scale, df, length)
summary(ratingsPerSound)  # mean 6.7 ratings/sound, range (1, 18)
nrow(ratingsPerSound) / (length(unique(df$sound)) * length(unique(df$scale)))  # 100% of combinations have occurred

# observed per prototype
obs = aggregate(response ~ cond + dB + scale + sound_base, df, mean)
head(obs)
ggplot(obs, aes(x = factor(dB), y = response, color = cond), inherit.aes = FALSE) +
  geom_violin(position = position_dodge(.5))

# use of rating scales
boxplot(respNorm ~ scale, df)
boxplot(respNorm ~ scale + subject, droplevels(df[df$subject %in% unique(df$subject)[1:10], ]))

# model
mod_file = '../mod/mod.RDS' 
if (file.exists(mod_file)) {
  mod = readRDS(mod_file)
} else {
  # set the difference b/w orig and mix to 0 when dB = 0 (these are the same sounds)
  brms::get_prior(bf(respNorm ~ cond * dB * scale + 
                       (cond * scale | subject + sound_base) + 
                       (scale|sound),
                     phi ~ (scale|subject)
  ),
  data = df, family = 'zero_one_inflated_beta')
  
  prior = c(
    set_prior(prior = 'normal(0, 2)', class = 'b'),
    set_prior(prior = 'constant(0)', class = 'b', coef = c(
      'condorig', 'condorig:scalearousal', 'condorig:scaleauthenticity', 
      'condorig:scalecontagion', 'condorig:scalepleasantness'
    ))
  )
  
  mod = brm(
    bf(respNorm ~ cond * dB * scale + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    prior = prior,
    chains = 3, cores = 3, warmup = 500, iter = 2000,
    # inits = 0,  # start all chains from the same par values
    init_r = 0.1,  # a better way to ensure convergence with a lot of random slopes - see https://discourse.mc-stan.org/t/model-with-many-correlated-varying-slopes-sampling-not-done/6189
    control = list(max_treedepth = 20)
  )  # 6 h for 11K * 2000 iter
  saveRDS(mod, mod_file)
}
# summary(mod)
# plot(mod)
# pp_check(mod)
# conditional_effects(mod)

# a quick look at fitted values
newdata = expand.grid(
  dB = unique(df$dB),  # seq(0, 18, length.out = 25), # 
  cond = levels(df$cond),
  scale = levels(df$scale),
  stringsAsFactors = FALSE
)
fit = fitted(mod, newdata = newdata, re_formula = NA, robust = TRUE) * 100  # to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')
df_pl = cbind(newdata, fit)
df_pl$condFmt = factor(as.character(names_conds[as.character(df_pl$cond)]), levels = as.character(names_conds))
df_pl$scaleFmt = factor(as.character(names_scales[as.character(df_pl$scale)]), levels = as.character(names_scales))
saveRDS(df_pl, '../data/fitted.RDS')

ggplot(df_pl, aes(x = dB, y = fit, ymin = lwr, ymax = upr, color = condFmt, linetype = condFmt, shape = condFmt, group = condFmt)) +
  # geom_point(size = 3, position = position_dodge(.25)) +
  # geom_line(size = 1, position = position_dodge(.1)) +
  geom_pointrange(position = position_dodge(1)) +
  geom_line(size = 1) +
  facet_wrap(~scaleFmt, nrow = 1) +
  xlab('Manipulation, dB') +
  ylab('Rating, %') +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = 'top',
        legend.title = element_blank())



## check whether speakerSex ets interact with the effect of manipulations. Because priors are a mess then, let's rerun the main model without them
mod_file0 = '../mod/mod0.RDS' 
if (file.exists(mod_file0)) {
  mod0 = readRDS(mod_file0)
} else {
  mod0 = brm(
    bf(respNorm ~ cond * dB * scale + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  ) 
  saveRDS(mod0, mod_file0)
}

# speakerSex
mod_file1a = '../mod/mod1a.RDS' 
if (file.exists(mod_file1a)) {
  mod1a = readRDS(mod_file1a)
} else {
  mod1a = brm(
    bf(respNorm ~ cond * dB * scale + speakerSex + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod1a, mod_file1a)
}

mod_file1 = '../mod/mod1.RDS' 
if (file.exists(mod_file1)) {
  mod1 = readRDS(mod_file1)
} else {
  mod1 = brm(
    bf(respNorm ~ cond * dB * scale * speakerSex + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod1, mod_file1)
}

loo(mod0, mod1a, mod1)
#       elpd_diff se_diff
# mod1a  0.0       0.0   
# mod0  -4.9       2.9   
# mod1  -8.5       4.9   


# corpus
mod_file2a = '../mod/mod2a.RDS' 
if (file.exists(mod_file2a)) {
  mod2a = readRDS(mod_file2a)
} else {
  mod2a = brm(
    bf(respNorm ~ cond * dB * scale + corpus + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod2a, mod_file2a)
}

mod_file2 = '../mod/mod2.RDS' 
if (file.exists(mod_file2)) {
  mod2 = readRDS(mod_file2)
} else {
  mod2 = brm(
    bf(respNorm ~ cond * dB * scale * corpus + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod2, mod_file2)
}

loo(mod0, mod2a, mod2)
#       elpd_diff se_diff
# mod0   0.0       0.0   
# mod2a -3.1       3.0   
# mod2  -3.5       4.8   

loo(mod2a, mod2)
#       elpd_diff se_diff
# mod2a  0.0       0.0   
# mod2  -0.3       4.9   

loo(mod, mod0, mod1a, mod1, mod2a, mod2)
#       elpd_diff se_diff
# mod1a  0.0       0.0   
# mod   -0.8       2.4   
# mod0  -4.9       2.9   
# mod2a -8.1       3.3   
# mod2  -8.4       5.0   
# mod1  -8.5       4.9   



# listener's sex
mod_file_sex = '../mod/mod_sex.RDS' 
if (file.exists(mod_file_sex)) {
  mod_sex = readRDS(mod_file_sex)
} else {
  mod_sex = brm(
    bf(respNorm ~ cond * dB * scale * sex + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod_sex, mod_file_sex)
}

# listener's age
df$age_norm = df$age / max(df$age)
mod_file_age = '../mod/mod_age.RDS' 
if (file.exists(mod_file_age)) {
  mod_age = readRDS(mod_file_age)
} else {
  mod_age = brm(
    bf(respNorm ~ cond * dB * scale * age_norm + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    chains = 2, cores = 2, warmup = 250, iter = 500,
    init_r = 0.1
  )  
  saveRDS(mod_age, mod_file_age)
}

loo(mod0, mod_sex, mod_age)
#         elpd_diff se_diff
# mod0     0.0       0.0   
# mod_age -4.4       4.6   
# mod_sex -5.6       4.3   
