# Script for modifying the degree of vocalizing during a manually annotated
# ingressive syllable in laughs. Algorithm: the ingressive region is mixed with
# noise (unvoiced but with the same formant structure) in different proportions,
# from 0 = original to 1 = only noise. Because breathing is seldom as loud as
# ingressive voiced sounds, we also penalize the amplitude as a linear function
# (in dB) of the proportion of noise

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(soundgen)

# annotations from-to code the ingressive regions
ann = read.csv('ingr_annotations.csv')
if (FALSE) {
  # descriptives of entire laughs
  ac = analyze('orig')
  d = ac$summary$duration
  ann = ann[ann$file %in% ac$summary$file, ]
  match(ann$file, ac$summary$file)
  mean(d); sd(d); range(d)
  # [1] 2.154906
  # [1] 0.5833108
  # [1] 0.93322 3.31220
  
  # descriptives of ingressive fragments
  dur = ann$to - ann$from
  mean(dur); sd(dur); range(dur)
  # [1] 436.0182
  # [1] 152.4821
  # [1]  237 1086
  
  for (i in 1:nrow(ann)) {
    temp = as.character(xfun::sans_ext(ann$file[i]))
    temp1 = unlist(strsplit(temp, '_'))
    ann$sound_base[i] = temp1[1]
    ann$speakerSex[i] = temp1[2]
    if (substr(temp, 1, 7) == 'session') {
      ann$corpus[i] = 'Wood 2020'
    } else {
      ann$corpus[i] = 'Anikin 2017'
    }
    if (ann$from[i] < 100) {
      ann$location[i] = 'start'
    } else if (d[i] * 1000 - ann$to[i] < 100) {
      ann$location[i] = 'end'
    } else {
      ann$location[i] = 'middle'
    }
  }
  table(ann$location)
  # end middle  start 
  #   7     40      8 

  add_vars = colnames(ac$summary)[-which(colnames(ac$summary) == 'file')]
  info = cbind(ann, ac$summary[match(ann$file, ac$summary$file), add_vars])
  write.csv(info, '../data/info.csv', row.names = FALSE)
}

# hyper-parameters
nMorphs = 3  # number of morphs - equidistant steps in proportions
loudness_penalty_dB = 18  # when only noise, reduce amplitude by ... dB


##############################################################################
#
# start synthesis
#
##############################################################################

files = list.files('orig', pattern = '.wav', full.names = TRUE)
prop_noise = seq(0, 1, length.out = nMorphs)
ampl_dB = seq(0, -loudness_penalty_dB, length.out = nMorphs)
ampl = 10 ^ (ampl_dB / 20)

time_start = proc.time()
for (f in 1:length(files)) {
  myfile = files[f]
  myfile_base = basename(myfile)
  myfile_wav = tuneR::readWave(myfile)
  samplingRate = myfile_wav@samp.rate
  myfile_vector = myfile_wav@left
  ann_f = ann[ann$file == myfile_base, ]
  minLen = 20 / 1000 * samplingRate   
  # don't bother attaching fragments < 20 ms (crossFade window)
  maxLen = length(myfile_vector)  # length of entire audio
  
  # find indices of "breaks" from egressive to ingressive and back
  break1 = max(1, round(samplingRate * ann_f$from / 1000))
  break2 = min(maxLen - 1, round(samplingRate * ann_f$to / 1000))
  
  egr1 = myfile_vector[1 : (break1 - 1)]
  ingr = myfile_vector[break1:break2]
  egr2 = myfile_vector[(break2 + 1) : length(myfile_vector)]
  
  # preserve original amplitude
  m = 2 ^ (myfile_wav@bit - 1)
  m_obs = max(abs(range(ingr)))
  len_ingr = break2 - break1 + 1
  
  # create pure noise (a bit longer than needed b/c transplantFormants shortens it)
  noise = runif(len_ingr + samplingRate * .01, -1, 1)
  # copy original formants and amplitude envelope
  noise_filt = transplantFormants(
    donor = ingr,
    recipient = noise,
    samplingRate = samplingRate,
    freqWindow = 200
  )
  noise_filt = transplantEnv(
    donor = ingr,
    recipient = noise_filt,
    samplingRateD = samplingRate
  )
  noise_filt = noise_filt / max(noise_filt) * m_obs
  noise_filt = noise_filt[1:len_ingr]
  # range(noise_filt)
  # playme(noise_filt, samplingRate)
  # spectrogram(noise_filt, samplingRate)
  
  # mix original + noise in different proportions
  rms_ingr_orig = sqrt(mean(ingr ^ 2))
  rms_noise = sqrt(mean(noise_filt ^ 2))
  for (i in 1:nMorphs) {
    ingr_i = noise_filt * prop_noise[i] + ingr * (1 - prop_noise[i])
    # equalize RMS in orig ingressive segment and in the synthesized noise
    rms_ingr = sqrt(mean(ingr_i ^ 2))
    ingr_i_ampl = ingr_i * rms_ingr_orig / rms_ingr # ampl[i]
    noise_ampl = noise_filt * rms_ingr_orig / rms_noise
    # playme(ingr_i, samplingRate)
    if (length(egr1) > minLen) {
      temp_mix = crossFade(egr1, ingr_i * ampl[i], samplingRate = samplingRate, crossLen = 20)
      temp_orig = crossFade(egr1, ingr * ampl[i], samplingRate = samplingRate, crossLen = 20)
      temp_noise = crossFade(egr1, noise_ampl * ampl[i], samplingRate = samplingRate, crossLen = 20)
    } else {
      temp_mix = ingr_i * ampl[i]
      temp_orig = ingr * ampl[i]
      temp_noise = noise_ampl * ampl[i]
    }
    if (length(egr2) > minLen) {
      temp_mix = crossFade(temp_mix, egr2, samplingRate = samplingRate, crossLen = 20)
      temp_orig = crossFade(temp_orig, egr2, samplingRate = samplingRate, crossLen = 20)
      temp_noise = crossFade(temp_noise, egr2, samplingRate = samplingRate, crossLen = 20)
    }
    filename_mix = paste0(
      'synth/', xfun::sans_ext(myfile_base), '_mix_', -ampl_dB[i], '.wav'  # round(prop_noise[i] * 100)
    )
    filename_orig = paste0(
      'synth/', xfun::sans_ext(myfile_base), '_orig_', -ampl_dB[i], '.wav'  # round(ampl[i] * 100)
    )
    filename_noise = paste0(
      'synth/', xfun::sans_ext(myfile_base), '_noise_', -ampl_dB[i], '.wav'
    )
    seewave::savewav(temp_mix, f = samplingRate, filename = filename_mix)
    seewave::savewav(temp_orig, f = samplingRate, filename = filename_orig)
    # seewave::savewav(temp_noise, f = samplingRate, filename = filename_noise)
  }
  reportTime(i = f, nIter = length(files), time_start = time_start)
  # lapply(out, playme, samplingRate = samplingRate)
  # playme(out[[10]], samplingRate)
  # spectrogram(out[[10]], samplingRate, yScale = 'bark')
}

spectrogram('synth', yScale = 'bark', savePlots = 'synth/plots')

# NB: "mix 0" is the same audio as "orig 100", and "mix 0" = "noise min"

# spectrogram('synth_6', yScale = 'bark', savePlots = '')
# spectrogram('synth_12', yScale = 'bark', savePlots = '')

if (FALSE) {
  ## a list of stimuli for javascript
  paste0(xfun::sans_ext(basename(files)), collapse = "', '")
  temp = expand.grid(dB = c('0', '9', '18'), cond = c('orig', 'mix'))
  temp$both = paste0(temp$cond, '_', temp$dB)
  paste(temp$both, collapse = "', '")
  
  # normalizeFolder('/home/allgoodguys/Documents/Research/experiments/ingress/html/examples', type = 'rms')  # let's just normalize by peak ampl instead of RMS because otherwise long quiet laughs will have very sharp, loud peaks
  
  # Convert to mp3 for the online experiment
  # pad with 100 ms of silence to avoid clicks (until already done during resynthesis)
  # for f in *.wav; do echo "Converting $f"; g=`basename $f .wav`; ffmpeg -i $f -aq 1 -af "adelay=100" -af "apad=pad_len=1600" mp3/$g.mp3 || echo FAILED; done
  
  
  # A few spectrograms for an illustration
  common_pars = list(yScale = 'bark', heights = c(1.5, 1), savePlots = '../pix/', width = 12, height = 12, units = 'cm', res = 600)
  do.call(spectrogram, c(list(x = 'synth/session12participant119-0122-3_M_mix_0.wav'), common_pars))
  do.call(spectrogram, c(list(x = 'synth/session12participant119-0122-3_M_mix_9.wav'), common_pars))
  do.call(spectrogram, c(list(x = 'synth/session12participant119-0122-3_M_mix_18.wav'), common_pars))
  do.call(spectrogram, c(list(x = 'synth/session12participant119-0122-3_M_orig_9.wav'), common_pars))
  do.call(spectrogram, c(list(x = 'synth/session12participant119-0122-3_M_orig_18.wav'), common_pars))
}

