# Script for modifying the degree of vocalizing during a manually annotated
# ingressive syllable(s. Algorithm: an ingressive fragment is mixed with noise
# (unvoiced but with the same formant structure) in different proportions, from
# 0 = original to 1 = only noise. Because breathing is seldom as loud as
# ingressive voiced sounds, we also penalize the amplitude as a linear function
# (in dB) of the proportion of noise. In the "original" condition, ingressive
# fragments are simply made quieter by the same amount as in the "noise"
# condition. Egressive and ingressive fragments are then cross-faded together.

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(soundgen)

# annotations from-to code the ingressive regions
ann = read.csv('cry_annotations.csv')

# hyper-parameters
nMorphs = 3  # number of morphs - equidistant steps in proportions
loudness_penalty_dB = 18  # when only noise, reduce amplitude by ... dB


##############################################################################
#
# start synthesis
#
##############################################################################

files = list.files('orig', pattern = '.wav', full.names = TRUE)
conds = c('orig', 'mix')
prop_noise = seq(0, 1, length.out = nMorphs)
ampl_dB = seq(0, -loudness_penalty_dB, length.out = nMorphs)
morphs = expand.grid(cond = conds, ampl_dB = ampl_dB)
morphs$ampl = 10 ^ (morphs$ampl_dB / 20)
morphs$prop_noise = rep(prop_noise, each = length(conds))

time_start = proc.time()
for (f in 1:length(files)) {
  myfile = files[f]
  myfile_base = basename(myfile)
  myfile_wav = tuneR::readWave(myfile)
  samplingRate = myfile_wav@samp.rate
  myfile_vector = myfile_wav@left
  ann_f = ann[ann$file == myfile_base, ]
  minLen = 20 / 1000 * samplingRate   
  # don't bother attaching fragments < 20 ms (crossFade window)
  maxLen = length(myfile_vector)  # length of entire audio
  
  # prepare segments
  segments = list()
  for (r in 1:nrow(ann_f)) {
    break1 = max(1, round(samplingRate * ann_f$from[r] / 1000))
    break2 = min(maxLen - 1, round(samplingRate * ann_f$to[r] / 1000))
    if (r == 1 & break1 > minLen) {
      segments[[length(segments) + 1]] = myfile_vector[1 : (break1 - 1)]
      names(segments)[length(segments)] = 'egr'
    }
    segments[[length(segments) + 1]] = myfile_vector[break1:break2]
    names(segments)[length(segments)] = 'ingr'
    if (r < nrow(ann_f)) {
      break_next = round(ann_f$from[r + 1] * samplingRate / 1000)
    } else {
      break_next = maxLen
    }
    if (break_next - break2 > minLen) {
      segments[[length(segments) + 1]] = myfile_vector[(break2 + 1) : (break_next - 1)]
      names(segments)[length(segments)] = 'egr'
    }
  }
  
  # preserve original amplitude
  m = 2 ^ (myfile_wav@bit - 1)
  
  # mix original + noise in different proportions
  for (i in 1:nrow(morphs)) {
    out = numeric(0)
    for (s in 1:length(segments)) {
      if (names(segments)[s] == 'egr') {
        # just use the original egressive segments
        new = segments[[s]]
      } else if (names(segments)[s] == 'ingr') {
        # ingressive - modify as per condition
        ingr = segments[[s]]
        m_obs = max(abs(range(ingr)))
        len_ingr = length(ingr)
        
        if (morphs$cond[i] == 'orig') {
          # just make the ingressive fragment quieter
          new = ingr * morphs$ampl[i]
        } else if (morphs$cond[i] == 'mix') {
          # morph to breathing noise 
          
          # make the noise a bit longer than needed b/c transplantFormants shortens it
          noise = runif(len_ingr + samplingRate * .01, -1, 1)
          
          # copy original formants and amplitude envelope
          noise_filt = transplantFormants(
            donor = ingr,
            recipient = noise,
            samplingRate = samplingRate,
            freqWindow = 200
          )
          noise_filt = transplantEnv(
            donor = ingr,
            recipient = noise_filt,
            samplingRateD = samplingRate
          )
          noise_filt = noise_filt / max(noise_filt) * m_obs
          noise_filt = noise_filt[1:len_ingr]
          # range(noise_filt)
          # playme(noise_filt, samplingRate)
          # spectrogram(noise_filt, samplingRate)
          
          # mix original + noise in different proportions
          rms_ingr_orig = sqrt(mean(ingr ^ 2))
          rms_noise = sqrt(mean(noise_filt ^ 2))
          ingr_i = noise_filt * morphs$prop_noise[i] + ingr * (1 - morphs$prop_noise[i])
          
          # equalize RMS in orig ingressive segment and in the synthesized noise
          rms_ingr = sqrt(mean(ingr_i ^ 2))
          # playme(ingr_i, samplingRate)
          new = ingr_i * rms_ingr_orig / rms_ingr * morphs$ampl[i]
        }
      }
      # add the new fragment to the growing sound
      out = crossFade(out, new, samplingRate = samplingRate, crossLen = 20)
    }
    # playme(out, samplingRate)
    # spectrogram(out, samplingRate)
    
    # save audio
    filename = paste0(
      'synth/', xfun::sans_ext(myfile_base), 
      '_', morphs$cond[i], '_', 
      -morphs$ampl_dB[i], '.wav'
    )
    seewave::savewav(out, f = samplingRate, filename = filename)
  }
  
  reportTime(i = f, nIter = length(files), time_start = time_start)
  # lapply(out, playme, samplingRate = samplingRate)
  # playme(out[[10]], samplingRate)
  # spectrogram(out[[10]], samplingRate, yScale = 'bark')
}

spectrogram('synth', yScale = 'bark', savePlots = 'synth/plots')

# NB: "mix 0" is the same audio as "orig 100", and "mix 0" = "noise min"

# spectrogram('synth_6', yScale = 'bark', savePlots = '')
# spectrogram('synth_12', yScale = 'bark', savePlots = '')

if (FALSE) {
  ## a list of stimuli for javascript
  paste0(xfun::sans_ext(basename(files)), collapse = "', '")
  temp = expand.grid(dB = c('0', '9', '18'), cond = c('orig', 'mix'))
  temp$both = paste0(temp$cond, '_', temp$dB)
  paste(temp$both, collapse = "', '")
  
  # normalizeFolder('/home/allgoodguys/Documents/Research/experiments/ingress/html/examples', type = 'rms')  # let's just normalize by peak ampl instead of RMS because otherwise long quiet laughs will have very sharp, loud peaks
  
  # Convert to mp3 for the online experiment
  # pad with 100 ms of silence to avoid clicks (until already done during resynthesis)
  # for f in *.wav; do echo "Converting $f"; g=`basename $f .wav`; ffmpeg -i $f -aq 1 -af "adelay=100" -af "apad=pad_len=1600" mp3/$g.mp3 || echo FAILED; done
}


if (FALSE) {
  # descriptives
  ac = analyze('orig')
  d = ac$summary$duration
  ann = ann[ann$file %in% ac$summary$file, ]
  match(ann$file, ac$summary$file)
  mean(d); sd(d); range(d)
# [1] 2.661849
# [1] 1.176304
# [1] 1.544444 6.797302
  
  # descriptives of ingressive fragments
  dur = ann$to - ann$from
  mean(dur); sd(dur); range(dur)
# [1] 560.84
# [1] 276.7491
# [1]  194 1224
  
  for (i in 1:nrow(ann)) {
    temp = as.character(xfun::sans_ext(ann$file[i]))
    temp1 = unlist(strsplit(temp, '_'))
    ann$sound_base[i] = temp1[1]
    ann$speakerSex[i] = temp1[2]
    if (substr(temp, 1, 7) == 'session') {
      ann$corpus[i] = 'Wood 2020'
    } else {
      ann$corpus[i] = 'Anikin 2017'
    }
    if (ann$from[i] < 100) {
      ann$location[i] = 'start'
    } else if (ac$summary$duration[ac$summary$file == ann$file[i]] * 1000 - ann$to[i] < 100) {
      ann$location[i] = 'end'
    } else {
      ann$location[i] = 'middle'
    }
  }
  table(ann$location)
   # end middle  start 
   #   6     10      9 
  
  ag = aggregate(from ~ file, ann, length)
  table(ag$from)
#    1  2  3 
# 18  2  1 

  add_vars = colnames(ac$summary)[-which(colnames(ac$summary) == 'file')]
  info = cbind(ann, ac$summary[match(ann$file, ac$summary$file), add_vars])
  write.csv(info, '../data/info.csv', row.names = FALSE)
}
