# Contrasts giving the effect sizes of manipulations - moans of pain and pleasure analyzed separately

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(brms)
library(ggplot2)
library(patchwork)
source('inm_helperFun.R')

# load data
df = read.csv('../data/inm.csv', stringsAsFactors = TRUE)
df$dB = as.numeric(df$dB)  # otherwise integer
table(df$cond, df$dB)

# observed 
ggplot(df, aes(dB, response, color = cond, shape = cond)) +
  geom_smooth(method = 'lm') +
  facet_wrap(~scale) +
  theme_bw()

boxplot(response ~ scale, df)
ggplot(df, aes(sound_base, response)) +
  geom_boxplot() +
  facet_wrap(~scale)

ag = aggregate(response ~ cond + dB + scale + realEm, df, median)
ggplot(ag, aes(dB, response, color = cond, shape = cond)) +
  geom_line() +
  facet_wrap(~realEm + scale, nrow = 2) +
  theme_bw()

# recognition "accuracy" - are moans of pleasure rated high on valence, and moan of pleasure low?
aggregate(response ~ realEm, df[df$scale == 'valence', ], mean)
ag = aggregate(response ~ realEm + sound_base, df[df$scale == 'valence', ], mean)
boxplot(response ~ realEm, ag)
table((ag$response > 50), ag$realEm)

# observed manipulation effects per prototype - very heterogeneous
ags = aggregate(response ~ scale + sound_base, df[df$realEm == 'pain' & df$dB == 0, ], mean)
ags$response18 = aggregate(response ~ scale + sound_base, df[df$realEm == 'pain' & df$dB == 18, ], mean)[, 3]
ags$effect = ags$response18 - ags$response
ggplot(ags, aes(scale, effect)) + geom_boxplot() 

ags = aggregate(response ~ scale + sound_base, df[df$realEm == 'pleasure' & df$dB == 0, ], mean)
ags$response18 = aggregate(response ~ scale + sound_base, df[df$realEm == 'pleasure' & df$dB == 18, ], mean)[, 3]
ags$effect = ags$response18 - ags$response
ggplot(ags, aes(scale, effect)) + geom_boxplot() 



### Contrasts for the model with *realEm interaction
# load the main model
mod = readRDS('../mod/mod_realEm.RDS')  

## Contrasts
# contrasts vs "dB = 0"
newdata = expand.grid(
  dB = unique(df$dB),
  cond = levels(df$cond),
  scale = levels(df$scale),
  realEm = levels(df$realEm),
  stringsAsFactors = FALSE
)
fit = fitted(mod, newdata = newdata, re_formula = NA, summary = FALSE) * 100
idx_zero_dB = which(newdata$dB == 0)
contr = droplevels(newdata[-idx_zero_dB, ])
for (i in 1:nrow(contr)) {
  idx = which(newdata$scale == contr$scale[i] &
                newdata$realEm == contr$realEm[i])
  idx_i = idx[which(newdata$cond[idx] == contr$cond[i] &
                      newdata$dB[idx] == contr$dB[i])]
  idx_0 = idx[which(newdata$dB[idx] == 0)] [1]  # both are identical - just "orig"
  d = fit[, idx_i] - fit[, idx_0]
  contr[i, c('fit', 'lwr', 'upr')] = quantile(d, probs = c(.5, .025, .975))
  if (mean(d) > 0) {
    contr$propPost[i] = mean(d > 0) * 100
  } else {
    contr$propPost[i] = mean(d < 0) * 100
  }
}
mean(contr$upr - contr$lwr)  # width of CI in %: 6.9%

# put dB = 0 back in, setting effect to 0
zero = newdata[idx_zero_dB, ]
zero[, c('fit', 'lwr', 'upr', 'propPost')] = 0
contr = rbind(contr, zero)

contr$condFmt = factor(as.character(names_conds[as.character(contr$cond)]), levels = as.character(names_conds))
contr$scaleFmt = factor(as.character(names_scales[as.character(contr$scale)]), levels = as.character(names_scales))
contr$realEmType = ifelse(contr$realEm == 'pleasure', 'Pleasure', 'Pain')
contr$realEmType = factor(contr$realEmType, levels = c('Pain', 'Pleasure'))
saveRDS(contr, '../data/contrasts_moans_realEm.RDS')

ggplot(contr, aes(x = dB, y = fit, ymin = lwr, ymax = upr, color = condFmt, linetype = condFmt, shape = condFmt, fill = condFmt, group = condFmt)) +
  geom_line(position = position_dodge(1)) +
  geom_point(size = 3, position = position_dodge(1)) +
  geom_errorbar(width = 0, linetype = 1, position = position_dodge(1)) +
  geom_hline(yintercept = 0, linetype = 3) +
  scale_color_manual(values = c('gray30', 'gray50')) +
  facet_wrap(~ realEmType + scaleFmt, nrow = 2) +
  xlab('Attenuation of ingressive fragment, dB') +
  ylab('Manipulation effect, %') +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = 'top',
        legend.title = element_blank())

# ggsave('../pix/contrasts_realEm.png', width = 20, height = 7, units = 'cm', dpi = 600)

# report CIs
contr$ci = soundgen:::reportCI(contr[, c('fit', 'lwr', 'upr')], 1)
contr[, c('cond', 'scale', 'dB', 'ci')]
contr[contr$dB == 18, c('realEm', 'cond', 'scale', 'dB', 'ci', 'propPost')]
contr[contr$dB == 18 & contr$propPost > 90, c('realEm', 'cond', 'scale', 'dB', 'ci', 'propPost')]

# saveRDS(contr, '../data/contrasts_moans_painPleasure.RDS')



## contrasts vs "dB = 0" per prototype
newdata = expand.grid(
  dB = unique(df$dB),
  cond = levels(df$cond),
  scale = levels(df$scale),
  realEm = levels(df$realEm),
  sound_base = levels(df$sound_base),
  stringsAsFactors = FALSE
)
fit = fitted(mod, newdata = newdata, re_formula = '~(cond * scale | sound_base)', summary = FALSE) * 100
idx_zero_dB = which(newdata$dB == 0)
contr = droplevels(newdata[-idx_zero_dB, ])
for (i in 1:nrow(contr)) {
  idx = which(newdata$scale == contr$scale[i] &
                newdata$val == contr$val[i] &
                newdata$sound_base == contr$sound_base[i])
  idx_i = idx[which(newdata$cond[idx] == contr$cond[i] &
                      newdata$dB[idx] == contr$dB[i])]
  idx_0 = idx[which(newdata$dB[idx] == 0)] [1]  # both are identical - just "orig"
  d = fit[, idx_i] - fit[, idx_0]
  contr[i, c('fit', 'lwr', 'upr')] = quantile(d, probs = c(.5, .025, .975))
  if (mean(d) > 0) {
    contr$propPost[i] = mean(d > 0) * 100
  } else {
    contr$propPost[i] = mean(d < 0) * 100
  }
}
mean(contr$upr - contr$lwr)  # width of CI in %: 11.2% (per sound_base)

# put dB = 0 back in, setting effect to 0
zero = newdata[idx_zero_dB, ]
zero[, c('fit', 'lwr', 'upr', 'propPost')] = 0
contr = rbind(contr, zero)

contr$condFmt = factor(as.character(names_conds[as.character(contr$cond)]), levels = as.character(names_conds))
contr$scaleFmt = factor(as.character(names_scales[as.character(contr$scale)]), levels = as.character(names_scales))
contr$valType = ifelse(contr$val == 1, 'Pleasure', 'Pain')
# contr$valType[contr$val == .5] = 'Ambiguous'
contr$valType = factor(contr$valType, levels = c('Pain', 'Pleasure'))

ggplot(contr, aes(x = dB, y = fit, ymin = lwr, ymax = upr, color = condFmt, linetype = condFmt, shape = condFmt, fill = condFmt, group = condFmt)) +
  # geom_violin(position = position_dodge(1)) +
  geom_point(size = 3, position = position_dodge(1)) +
  geom_hline(yintercept = 0, linetype = 3) +
  scale_color_manual(values = c('gray30', 'gray50')) +
  facet_wrap(~ valType + scaleFmt, ncol = 5) +
  xlab('Attenuation of ingressive fragment, dB') +
  ylab('Manipulation effect, %') +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = 'top',
        legend.title = element_blank())
# looks like quite strong shrinkage