# Main model for the effect of cond and dB 

setwd(dirname(rstudioapi::getActiveDocumentContext()$path))

library(brms)
library(ggplot2)
source('inm_helperFun.R')

df = read.csv('../data/inm.csv', stringsAsFactors = TRUE)
df$scale = factor(df$scale, levels = names(names_scales))  
# to have scales in a standard order for prior specification (everything is compared to level1)

# model for all moans
mod_file0 = '../mod/mod0.RDS' 
if (file.exists(mod_file0)) {
  mod0 = readRDS(mod_file0)
} else {
  # set the difference b/w orig and mix to 0 when dB = 0 (these are the same sounds)
  brms::get_prior(bf(respNorm ~ cond * dB * scale + 
                       (cond * scale | subject + sound_base) + 
                       (scale|sound),
                     phi ~ (scale|subject)
  ),
  data = df, family = 'zero_one_inflated_beta')
  
  prior = c(
    set_prior(prior = 'normal(0, 2)', class = 'b'),
    set_prior(prior = 'constant(0)', class = 'b', coef = c(
      'condorig', 'condorig:scalearousal', 'condorig:scaleauthenticity', 
      'condorig:scalecontagion', 'condorig:scaledisturbing'
    ))
  )
  
  mod0 = brm(
    bf(respNorm ~ cond * dB * scale + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    prior = prior,
    chains = 3, cores = 3, warmup = 500, iter = 2000,
    # inits = 0,  # start all chains from the same par values
    init_r = 0.1,  # a better way to ensure convergence with a lot of random slopes - see https://discourse.mc-stan.org/t/model-with-many-correlated-varying-slopes-sampling-not-done/6189
    control = list(max_treedepth = 20)
  )  # 2 h for 5K * 2000 iter
  saveRDS(mod0, mod_file0)
}
# summary(mod0)
# plot(mod0)
# pp_check(mod0)
# conditional_effects(mod0)


# model with interaction with *realEm
mod_file = '../mod/mod_realEm.RDS' 
if (file.exists(mod_file)) {
  mod = readRDS(mod_file)
} else {
  # set the difference b/w orig and mix to 0 when dB = 0 (these are the same sounds)
  brms::get_prior(bf(respNorm ~ cond * dB * scale * realEm + 
                       (cond * scale | subject + sound_base) + 
                       (scale|sound),
                     phi ~ (scale|subject)
  ),
  data = df, family = 'zero_one_inflated_beta')
  
  prior = c(
    set_prior(prior = 'normal(0, 2)', class = 'b'),
    set_prior(prior = 'constant(0)', class = 'b', coef = c(
      'condorig', 
      'condorig:scalearousal', 
      'condorig:scaleauthenticity', 
      'condorig:scalecontagion', 
      'condorig:scaledisturbing',
      'condorig:realEmpleasure', 
      'condorig:scalearousal:realEmpleasure', 
      'condorig:scaleauthenticity:realEmpleasure', 
      'condorig:scalecontagion:realEmpleasure', 
      'condorig:scaledisturbing:realEmpleasure'
    ))
  )
  
  mod = brm(
    bf(respNorm ~ cond * dB * scale * realEm + 
         (cond * scale | subject + sound_base) + 
         (scale|sound),
       phi ~ (scale|subject)
    ),
    data = df, family = 'zero_one_inflated_beta', 
    prior = prior,
    chains = 3, cores = 3, warmup = 500, iter = 2000,
    # inits = 0,  # start all chains from the same par values
    init_r = 0.1,  # a better way to ensure convergence with a lot of random slopes - see https://discourse.mc-stan.org/t/model-with-many-correlated-varying-slopes-sampling-not-done/6189
    control = list(max_treedepth = 20)
  )  # 2 h for 5K * 2000 iter
  saveRDS(mod, mod_file)
}
# summary(mod)
# plot(mod)
# pp_check(mod)
# conditional_effects(mod)

# loo(mod0, mod)  # about the same
# Model comparisons:
#      elpd_diff se_diff
# mod0  0.0       0.0   
# mod  -4.7       4.5   

# a quick look at fitted values
newdata = expand.grid(
  dB = unique(df$dB),  # seq(0, 18, length.out = 25), # 
  cond = levels(df$cond),
  scale = levels(df$scale),
  realEm = levels(df$realEm),
  stringsAsFactors = FALSE
)
fit = fitted(mod, newdata = newdata, re_formula = NA, robust = TRUE) * 100  # to %
colnames(fit) = c('fit', 'se', 'lwr', 'upr')

df_pl = cbind(newdata, fit)
df_pl$condFmt = factor(as.character(names_conds[as.character(df_pl$cond)]), levels = as.character(names_conds))
df_pl$scaleFmt = factor(as.character(names_scales[as.character(df_pl$scale)]), levels = as.character(names_scales))
saveRDS(df_pl, '../data/fitted.RDS')

ggplot(df_pl, aes(x = dB, y = fit, ymin = lwr, ymax = upr, color = condFmt, linetype = condFmt, shape = condFmt, group = condFmt)) +
  geom_pointrange(position = position_dodge(1)) +
  geom_line(size = 1) +
  facet_wrap(~realEm + scaleFmt, nrow = 2) +
  xlab('Manipulation, dB') +
  ylab('Rating, %') +
  theme_bw() +
  theme(panel.grid = element_blank(),
        legend.position = 'top',
        legend.title = element_blank())
